require 'refinery'

module Refinery
  module Gallery
    class Engine < Rails::Engine
      initializer "static assets" do |app|
        app.middleware.insert_after ::ActionDispatch::Static, ::ActionDispatch::Static, "#{root}/public"
      end

      config.to_prepare do
        Article.module_eval do
          has_many :galleries
          has_many :images, :through => :galleries, :order => 'position ASC'
          # accepts_nested_attributes_for MUST come before def images_attributes=
          # this is because images_attributes= overrides accepts_nested_attributes_for.
          accepts_nested_attributes_for :images, :allow_destroy => false

          def images_attributes=(data)
            Gallery.delete_all(:article_id => self.id)

            (0..(data.length-1)).each do |i|
              unless (image_data = data[i.to_s]).nil? or image_data['id'].blank?
                gallery = self.gallery.new(:image_id => image_data['id'].to_i, :position => i)
                # Add caption if supported
                if RefinerySetting.find_or_set(:captions, false)
                  gallery.caption = image_data['caption']
                end
                self.gallery << gallery
              end
            end
          end

          def caption_for_image_index(index)
            self.gallery[index].try(:caption).presence || ""
          end
        end
      end

      config.after_initialize do
        ::Refinery::Pages::Tab.register do |tab|
          tab.name = "images"
          tab.partial = "/admin/articles/tabs/images"
        end
        ::Refinery::Plugin.register do |plugin|
          plugin.name = "gallery"
          plugin.hide_from_menu = true
        end
      end
    end
  end
end
